@@ -17,34 +17,36 @@
 #define _GNU_SOURCE 1
 #define _POSIX_PTHREAD_SEMANTICS 1
 #define _TANDEM_SOURCE 1
+#define HAVE_INFINITY 1
+#define HAVE_NAN 1
 #define RUBY_SYMBOL_EXPORT_BEGIN _Pragma("GCC visibility push(default)")
 #define RUBY_SYMBOL_EXPORT_END _Pragma("GCC visibility pop")
 #define HAVE_DIRENT_H 1
 #define HAVE__BOOL 1
 #define HAVE_STDBOOL_H 1
 #define HAVE_SYS_WAIT_H 1
-#define HAVE_LIMITS_H 1
-#define HAVE_SYS_FILE_H 1
-#define HAVE_SYS_IOCTL_H 1
-#define HAVE_SYS_SYSCALL_H 1
-#define HAVE_FCNTL_H 1
-#define HAVE_SYS_FCNTL_H 1
-#define HAVE_SYS_PARAM_H 1
-#define HAVE_SYS_SELECT_H 1
-#define HAVE_SYS_TIME_H 1
-#define HAVE_SYS_TIMES_H 1
-#define HAVE_PWD_H 1
-#define HAVE_GRP_H 1
 #define HAVE_A_OUT_H 1
-#define HAVE_UTIME_H 1
-#define HAVE_SYS_RESOURCE_H 1
+#define HAVE_GRP_H 1
+#define HAVE_FCNTL_H 1
 #define HAVE_FLOAT_H 1
 #define HAVE_IEEEFP_H 1
 #define HAVE_LANGINFO_H 1
+#define HAVE_LIMITS_H 1
 #define HAVE_LOCALE_H 1
-#define HAVE_TIME_H 1
+#define HAVE_PWD_H 1
+#define HAVE_SYS_FCNTL_H 1
+#define HAVE_SYS_FILE_H 1
+#define HAVE_SYS_IOCTL_H 1
+#define HAVE_SYS_PARAM_H 1
+#define HAVE_SYS_RESOURCE_H 1
+#define HAVE_SYS_SELECT_H 1
 #define HAVE_SYS_SOCKET_H 1
+#define HAVE_SYS_SYSCALL_H 1
+#define HAVE_SYS_TIME_H 1
+#define HAVE_SYS_TIMES_H 1
 #define HAVE_SYS_UIO_H 1
+#define HAVE_TIME_H 1
+#define HAVE_UTIME_H 1
 #define HAVE_GMP_H 1
 #define HAVE_LIBGMP 1
 #define HAVE_TYPEOF 1
@@ -55,7 +57,6 @@
 #define SIZEOF_LONG 8
 #define SIZEOF_LONG_LONG 8
 #define SIZEOF___INT64 0
-#define SIZEOF___INT128 16
 #define SIZEOF_OFF_T 8
 #define SIZEOF_VOIDP 8
 #define SIZEOF_FLOAT 4
@@ -117,14 +118,16 @@
 #define HAVE_VA_ARGS_MACRO 1
 #define NORETURN(x) __attribute__ ((noreturn)) x
 #define DEPRECATED(x) __attribute__ ((deprecated)) x
+#define DEPRECATED_TYPE(mesg,x) __attribute__ ((deprecated mesg)) x
 #define NOINLINE(x) __attribute__ ((noinline)) x
-#define FUNC_CDECL(x) __attribute__ ((cdecl)) x
+#define WEAK(x) __attribute__ ((weak)) x
+#define HAVE_FUNC_WEAK 1
 #define HAVE_ATTRIBUTE_FUNCTION_ALIAS 1
 #define RUBY_ALIAS_FUNCTION_TYPE(type, prot, name, args) type prot __attribute__((alias(#name)));
 #define RUBY_ALIAS_FUNCTION_VOID(prot, name, args) RUBY_ALIAS_FUNCTION_TYPE(void, prot, name, args)
-#define HAVE_GCC_ATOMIC_BUILTINS 1
 #define HAVE_GCC_SYNC_BUILTINS 1
 #define RUBY_FUNC_EXPORTED __attribute__ ((visibility("default"))) extern
+#define ENUM_OVER_INT 1
 #define HAVE_DECL_SYS_NERR 1
 #define HAVE_DECL_GETENV 1
 #define SIZEOF_SIZE_T 8
@@ -171,12 +174,6 @@
 #define SIZEOF_INT64_T 8
 #define HAVE_UINT64_T 1
 #define SIZEOF_UINT64_T 8
-#define HAVE_INT128_T 1
-#define int128_t __int128
-#define SIZEOF_INT128_T SIZEOF___INT128
-#define HAVE_UINT128_T 1
-#define uint128_t unsigned __int128
-#define SIZEOF_UINT128_T SIZEOF_UNSIGNED___INT128
 #define HAVE_INTPTR_T 1
 #define SIZEOF_INTPTR_T 8
 #define HAVE_UINTPTR_T 1
@@ -192,6 +189,7 @@
 #define HAVE_CRYPT 1
 #define HAVE_DUP2 1
 #define HAVE_ERF 1
+#define HAVE_EXPLICIT_BZERO 1
 #define HAVE_FFS 1
 #define HAVE_FINITE 1
 #define HAVE_FLOCK 1
@@ -215,6 +213,7 @@
 #define HAVE_WORKING_FORK 1
 #define HAVE___SYSCALL 1
 #define HAVE__LONGJMP 1
+#define HAVE__SETJMP 1
 #define HAVE_ATAN2L 1
 #define HAVE_ATAN2F 1
 #define HAVE_CHROOT 1
@@ -263,6 +262,8 @@
 #define HAVE_WRITEV 1
 #define HAVE_MEMRCHR 1
 #define HAVE_MEMMEM 1
+#define HAVE_MKFIFO 1
+#define HAVE_MKNOD 1
 #define HAVE_MKTIME 1
 #define HAVE_PIPE2 1
 #define HAVE_POLL 1
@@ -304,14 +305,10 @@
 #define HAVE_UTIMES 1
 #define HAVE_WAIT4 1
 #define HAVE_WAITPID 1
-#define HAVE_BUILTIN___BUILTIN_BSWAP16 1
-#define HAVE_BUILTIN___BUILTIN_BSWAP32 1
-#define HAVE_BUILTIN___BUILTIN_BSWAP64 1
 #define HAVE_BUILTIN___BUILTIN_CLZ 1
 #define HAVE_BUILTIN___BUILTIN_CLZL 1
 #define HAVE_BUILTIN___BUILTIN_CLZLL 1
 #define HAVE_BUILTIN___BUILTIN_CHOOSE_EXPR 1
-#define HAVE_BUILTIN___BUILTIN_CHOOSE_EXPR_CONSTANT_P 1
 #define HAVE_BUILTIN___BUILTIN_TYPES_COMPATIBLE_P 1
 #define ATAN2_INF_C99 1
 #define HAVE_CLOCK_GETRES 1
@@ -340,15 +337,20 @@
 #define HAVE_PTHREAD_CONDATTR_SETCLOCK 1
 #define HAVE_PTHREAD_CONDATTR_INIT 1
 #define HAVE_PTHREAD_SIGMASK 1
+#define HAVE_PTHREAD_SET_NAME_NP 1
 #define HAVE_PTHREAD_ATTR_INIT 1
+#define SET_CURRENT_THREAD_NAME(name) pthread_set_name_np(pthread_self(), name)
+#define SET_ANOTHER_THREAD_NAME(thid,name) pthread_set_name_np(thid, name)
+#define UCONTEXT_IN_SIGNAL_H 1
+#define DEFINE_MCONTEXT_PTR(mc, uc) mcontext_t *mc = &(uc)->uc_mcontext
 #define USE_ELF 1
 #define HAVE_ELF_ABI_H 1
 #define DLEXT_MAXLEN 3
 #define DLEXT ".so"
 #define LIBDIR_BASENAME "lib"
-#define RUBY_SETJMP(env) sigsetjmp((env),0)
-#define RUBY_LONGJMP(env,val) siglongjmp((env),val)
-#define RUBY_JMP_BUF sigjmp_buf
+#define RUBY_SETJMP(env) _setjmp((env))
+#define RUBY_LONGJMP(env,val) _longjmp((env),val)
+#define RUBY_JMP_BUF jmp_buf
 #define HAVE_PTHREAD_H 1
 #define RUBY_PLATFORM "x86_64-openbsd"
 #endif /* INCLUDE_RUBY_CONFIG_H */
