@@ -1,3 +1,4 @@
+# frozen_string_literal: false
 require 'rubygems/command'
 require 'rubygems/install_update_options'
 require 'rubygems/dependency_installer'
@@ -275,7 +276,7 @@ to write the specification by hand.  For example:
       gem = fetcher.download_to_cache dependency
     end
 
-    inst = Gem::Installer.new gem, options
+    inst = Gem::Installer.at gem, options
     inst.install
 
     require 'rubygems/dependency_installer'
